let lastDailyTCVStats = [];
let lastSaleDistribution = [];
let lastDailyStats = [];
let lastMonthlyStats = [];

function resetLastStatsVariables() {
  lastDailyTCVStats = [];
  lastSaleDistribution = [];
  lastDailyStats = [];
  lastMonthlyStats = [];
}

function initializeStats(data) {
  if (!data.hasOwnProperty('proStats')) {
    return;
  }
  let dailyStats = data.proStats.dailyStats.reverse();
  let monthlyStats = data.proStats.monthlyStats.reverse();

  let dailyStatsUnion = _.intersectionWith(dailyStats, lastDailyStats, _.isEqual);
  let monthlyStatsUnion = _.intersectionWith(monthlyStats, lastMonthlyStats, _.isEqual);
  let saleDistributionUnion = _.intersectionWith(data.proStats.saleDistribution, lastSaleDistribution, _.isEqual);
  let dailyTCVStatsUnion = _.intersectionWith(data.proStats.dailyTCVStats, lastDailyTCVStats, _.isEqual);

  if (dailyStatsUnion.length !== dailyStats.length) {
    prepareProfitCharts(dailyStats, 'dailyProfitsChart');
    prepareEarnedVsSpentCharts(dailyStats, 'dailySpentEarnedChart');
    prepareBuysVsSalesCharts(dailyStats, 'dailyBuySalesChart');
    lastDailyStats = dailyStats;
  }

  if (monthlyStatsUnion.length !== monthlyStats.length) {
    prepareProfitCharts(monthlyStats, 'monthlyProfitsChart');
    prepareEarnedVsSpentCharts(monthlyStats, 'monthlySpentEarnedChart');
    prepareBuysVsSalesCharts(monthlyStats, 'monthlyBuySalesChart');
    lastMonthlyStats = monthlyStats;
  }

  if (saleDistributionUnion.length !== data.proStats.saleDistribution.length) {
    prepareSaleDistributionChart(data.proStats.saleDistribution, 'saleDistribution');
    lastSaleDistribution = data.proStats.saleDistribution;
  }

  if (dailyTCVStatsUnion.length !== data.proStats.dailyTCVStats.length) {
    prepareTCVChart(data.proStats.dailyTCVStats, 'TCVChart');
    lastDailyTCVStats = data.proStats.dailyTCVStats;
  }
}

function prepareTCVChart(data, element) {
    let labels = []
    let minTCV = []
    let maxTCV = []

    let sortedData = _.orderBy(data, 'order', 'asc');

    for (let i = 0; i < sortedData.length; i++) {
      let date = sortedData[i].date;
      date = date.split('-').join('/');
      date = date.split('/');
      date = date[1] + '/' + date[0] + '/' + date[2];
      date = DateTimeHelper.formatDate(new Date(date));
      date = date.split(' ')[0];
      labels.push(date);

      minTCV.push(parseFloat((sortedData[i].minTCV).toFixed(8)));
      maxTCV.push(parseFloat((sortedData[i].maxTCV).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type":"line",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.minTCV'),
                        "data": minTCV,
                        "fill": false,
                        "borderColor":"rgb(75, 192, 192)",
                        "lineTension":0.4,
                        pointStyle: 'line'
                    },
                    {
                        "label": i18next.t('statsPage.maxTCV'),
                        "data": maxTCV,
                        "fill": false,
                        "borderColor":"#ef5350",
                        "lineTension":0.4,
                        pointStyle: 'line'
                    }
                ],
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {
                    "mode": "index",
                    "intersect": false,
                },
                "hover": {
                    "mode": "index",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "beginAtZero": false,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            },
                            suggestedMin: _.min(minTCV) - (0.01 / _.min(minTCV)),
                            suggestedMax: _.max(maxTCV) + (0.01 * _.max(maxTCV))
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });
}

function prepareSaleDistributionChart(data, element) {
    let labels = [], values = []
    data.forEach(element => {
        labels.push(element.name)
        values.push(element.value)
    });

    new Chart(document.getElementById(element),
        {
            "type":"doughnut",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.SalesDistribution'),
                        "data": values,
                        "backgroundColor":[
                            "rgb(5, 177, 111)",
                            "#96efbf",
                            "rgb(216, 83, 83)",
                            "#f66000",
                            "#6CB46C",
                            "#7bbbff",
                            "rgba(255, 215, 0, 0.75)"
                        ],
                        borderColor: [
                            "rgb(5, 177, 111)",
                            "#96efbf",
                            "rgb(216, 83, 83)",
                            "#f66000",
                            "#6CB46C",
                            "#7bbbff",
                            "rgba(255, 215, 0, 0.75)"
                        ]
                    }
                ]
            },
            "options": {
                "maintainAspectRatio": false,
                "plugins": {
                    "labels": [
                        {
                            "render": 'percentage',
                            "fontColor": "#000",
                            "precision": 0,
                            "showZero": false,
                            showActualPercentages: true,
                        }
                    ]
                },
                legend: {
                    display: true,
                    position: "right",
                    fullWidth: false,
                    labels: {
                      boxWidth: 1,
                      padding: 10,
                      fontSize: 13
                    }
                  },
                "animation": false
            }
        });
}

function prepareBuysVsSalesCharts(sales, element) {
    let labels = []
    let sells = []
    let buys = []

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (var i = 0; i < sortedData.length; i++) {
        let date;
        if(element === 'monthlyBuySalesChart') {
            date = sortedData[i].month.split('-').join('/');
        } else {
          date = sortedData[i].date.split('-').join('/');
          date = date.split('/');
          date = date[1] + '/' + date[0] + '/' + date[2];
          date = DateTimeHelper.formatDate(new Date(date));
          date = date.split(' ')[0];
        }
        labels.push(date);

        sells.push((sortedData[i].totalSales));
        buys.push((sortedData[i].totalBuys));
    }

    new Chart(document.getElementById(element),
        {
            "type":"bar",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Sales'),
                        "data": sells,
                        "backgroundColor":"rgb(75, 192, 192)",
                        "borderColor":"rgb(75, 192, 192)",
                        pointStyle: 'line'
                    },
                    {
                        "label": i18next.t('statsPage.Buys'),
                        "data": buys,
                        "backgroundColor":"#ef5350",
                        "borderColor":"#ef5350",
                        pointStyle: 'line'
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {

                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            beginAtZero: true
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareEarnedVsSpentCharts(sales, element) {
    let labels = []
    let spent = []
    let earned = []

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (var i = 0; i < sortedData.length; i++) {
        let date;
        if(element === 'monthlySpentEarnedChart') {
          date = sortedData[i].month.split('-').join('/');
        } else {
          date = sortedData[i].date.split('-').join('/');
          date = date.split('/');
          date = date[1] + '/' + date[0] + '/' + date[2];
          date = DateTimeHelper.formatDate(new Date(date));
          date = date.split(' ')[0];
        }
        labels.push(date);

        spent.push(parseFloat((sortedData[i].totalBoughtCost).toFixed(8)));
        earned.push(parseFloat((sortedData[i].totalSoldValue).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type":"bar",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Spent'),
                        "data": spent,
                        "backgroundColor":"rgb(75, 192, 192)",
                        "borderColor":"rgb(75, 192, 192)",
                        pointStyle: 'line'
                    },
                    {
                        "label": i18next.t('statsPage.Earned'),
                        "data": earned,
                        "backgroundColor":"#ef5350",
                        "borderColor":"#ef5350",
                        pointStyle: 'line'
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {

                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            beginAtZero: true,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareProfitCharts(sales, element) {
    let labels = [];
    let profits = [];

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (var i = 0; i < sortedData.length; i++) {
        let date;
        if(element === 'monthlyProfitsChart') {
          date = sortedData[i].month.replace('-', '/');
        } else {
          date = sortedData[i].date.split('-').join('/');
          date = date.split('/');
          date = date[1] + '/' + date[0] + '/' + date[2];
          date = DateTimeHelper.formatDate(new Date(date));
          date = date.split(' ')[0];
        }
        labels.push(date);

        profits.push(parseFloat((sortedData[i].totalProfitCurrency).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type":"line",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Profits'),
                        "data": profits,
                        "fill":false,
                        "borderColor":"rgb(75, 192, 192)",
                        "lineTension":0.4,
                        pointStyle: 'line'
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {
                    "mode": "nearest",
                    "intersect": false,
                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "beginAtZero": true,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });

}


$( "#swalSaleDistribution" ).on( "click", swalSaleDistribution );

function swalSaleDistribution() {
    let content = document.createElement('div');
    content.innerHTML = 'TP = ' + i18next.t('statsPage.TakeProfit') + '<br>' +
        'TPS = ' + i18next.t('statsPage.TakeProfitSafety') + '<br>' +
        'TSL = ' + i18next.t('statsPage.TrailingStopLoss') + '<br>' +
        'M = ' + i18next.t('statsPage.Manual') + '<br>' +
        'MP = ' + i18next.t('statsPage.MaxProfit') + '<br>' +
        'G = ' + i18next.t('statsPage.Gain') + '<br>';

    swal({
        title: 'Sales Distribution Abbreviations',
        content: content,
    })
}
